/* Cytec Matrix Test Program for LAN with UNIX o/s         */

#include <stdio.h>      /* for printf() and fprintf() */
#include <sys/socket.h> /* for socket(), connect(), send(), and recv() */
#include <arpa/inet.h>  /* for sockaddr_in and inet_addr() */
#include <stdlib.h>     /* for atoi() and exit() */
#include <string.h>     /* for memset() */
#include <unistd.h>     /* for close() */

int init_LAN(int sock);   
int point_ops(int sock,int cmd, int mtx, int mod, int rly);
int matrix_clear(int sock);
void DieWithError(char *errorMessage);


#define MAX_MTX	1
#define MAX_MOD	4
#define MAX_RLY 12

int main()
{
	int sock;
	char servIP[40];
	struct sockaddr_in servAddr; /* Echo server address */   
	unsigned short servPort;     /* Echo server port */ 
    int mtx, mod, rly,status; 

	servPort = 8080;
	strcpy(servIP,"10.0.0.26");
	
	/* Create a reliable, stream socket using TCP */        
	if ((sock = socket(PF_INET, SOCK_STREAM, IPPROTO_TCP))<0)
		DieWithError("socket() failed");   
	
	/* Construct the server address structure */ 		   
	memset(&servAddr, 0, sizeof(servAddr));     /* Zero out structure */
    servAddr.sin_family      = AF_INET;             /* Internet address family */
    servAddr.sin_addr.s_addr = inet_addr(servIP);   /* Server IP address */
    servAddr.sin_port        = htons(servPort); /* Server port */
	
    /* Establish the connection to the server */
    if (connect(sock, (struct sockaddr *) &servAddr, sizeof(servAddr))<0)
			DieWithError("connect() failed");  
	
	/* Initialize Device using init_LAN Function */
	init_LAN(sock);
	
	/* Send Clear Command to Device with matrix_clear Function*/
	if ((status = matrix_clear(sock)) != 48)
		printf("Error clearing device/n");
	
	/* Simple looping through switchpoints */
	
	for(mtx=0; mtx<MAX_MTX; mtx++)
	{
		for (mod=0; mod<MAX_MOD; mod++) 
		{
				for (rly=0; rly<MAX_RLY; rly++) 
				{  
					if (((status = point_ops(sock,'L',mtx,mod,rly))) !=49)
						printf("Error point %d %d %d not closed\n",mtx,mod,rly);
					else 
						printf("Latched point %d %d\n",mod, rly);
					
					if (((status = point_ops(sock,'U',mtx,mod,rly))) !=48)
						printf("Error point %d %d %d not open\n",mtx,mod,rly);
					else
						printf("Unlatched point %d %d\n",mod, rly); 
				}
		}
	}
	close(sock);  
	return 0;
}
/*--------------Initialize---------------------------*/

int init_LAN(int sock)
{
	char rcvString[40];     /* Buffer for device response */
    int rcvStringLen;      /* Length of device response */
	void DieWithError(char *errorMessage);
	/* Initialize Device */		 
	if ((send(sock, "E0 73;V0 73;TCPANSWERBACK 1\n", 28, 0)) != 28)
	{
			DieWithError("send() failed");   
	}
	sleep(1);	  /* Wait for Response from Device */
	if ((rcvStringLen = recv(sock, rcvString, 9, 0)) < 9)
	{
			DieWithError("recv() failed or connection closed prematurely");
	}
	rcvString[rcvStringLen] = '\0';
	return 0;
}

/*-----------------Clear Matrix----------------------*/

int matrix_clear(int sock)
{

	char rcvString[8];     /* Buffer for device response */
    int rcvStringLen;      /* Length of device response */
	
    if ((send(sock,"C\n",2,0)) != 2)  
			DieWithError("send() failed");
			
	sleep(1);   /* Wait for Response
	
	/* Receive Response from Device */
	if ((rcvStringLen = recv(sock, rcvString, 10, 0)) <= 0)
			DieWithError("recv() failed or connection closed prematurely"); 
    rcvString[rcvStringLen] = '\0';
	int status = rcvString[0] & 0x3f;

	return status;  
}
/*-----------Switchpoint Operation------------------  */

int point_ops(int sock,int cmd, int mtx, int mod, int rly)
{
	char cmd_str[40];	   /* Formatted command string */
	char rcvString[8];     /* Buffer for device response */
    int rcvStringLen;      /* Length of device response */

	/* Format String */
	sprintf(cmd_str,"%c%d %d %d\n",cmd,mtx,mod,rly);
	
	/* Send Command to Device */
	if ((send(sock,cmd_str,strlen(cmd_str),0)) != strlen(cmd_str))  
			DieWithError("send() failed");
			
	sleep(1);   /* Wait for Response
	
	/* Receive Response from Device */
	if ((rcvStringLen = recv(sock, rcvString, 10, 0)) <= 0)
			DieWithError("recv() failed or connection closed prematurely"); 
    rcvString[rcvStringLen] = '\0';
	int status = rcvString[0] & 0x3f;

	return status;
}

/*------------Error Handling Function---------------------*/
	
 void DieWithError(char *errorMessage)
{
    perror(errorMessage);
	getchar();
    exit(1);
}
